from .enums import *
from .exceptions import DaftException
from .listing import Listing
from .request import Request


class Daft(object):
    def __init__(self, xml_url=None, debug=False):
        self._base = 'http://www.daft.ie/'
        self._debug = debug
        self._xml_url = xml_url
        self._sale_agreed = False
        self._open_viewing = False
        self._offset = 0
        self._keywords = []
        self._query_params = ""
        self._commercial_property_type = ""
        self._price = ""
        self._listing_type = SaleType.PROPERTIES
        self._student_accommodation_type = StudentAccommodationType.ANY
        self._area = None
        self._county = None
        self._min_price = None
        self._max_price = None
        self._min_beds = None
        self._max_beds = None
        self._sort_by = None
        self._sort_order = None
        self._commercial_min_size = None
        self._commercial_max_size = None
        self._university = None
        self._result_url = None

    def set_result_url(self, result_url):
        """
        Pass result url to scrape search results.
        :param result_url: string
        :return:
        """
        self._result_url = result_url

    def set_address(self, address):
        """
        Set the address.
        :param address:
        """
        self._query_params += str(QueryParam.ADVANCED) + str(QueryParam.ADDRESS) + address.replace(" ", "+").lower()

    def set_min_lease(self, min_lease):
        """
        Set the minimum lease period in months.
        :param min_lease: int
        """
        self._query_params += str(QueryParam.MIN_LEASE) + str(min_lease)

    def set_added_since(self, added):
        """
        Set this to retrieve ads that are a given number of days old.
        For example to retrieve listings that have been been added a week ago: set_added_since(7)
        :param added: int
        """
        self._query_params += str(QueryParam.DAYS_OLD) + str(added)

    def set_max_lease(self, max_lease):
        """
        Set the maximum lease period in months.
        :param max_lease: int
        """
        self._query_params += str(QueryParam.MAX_LEASE) + str(max_lease)

    def set_availability(self, availability):
        """
        Set the maximum lease period in months.
        :param availability:
        """
        if availability >= 5:
            availability = '5%2B'
        self._query_params += str(QueryParam.AVALIABILITY) + str(availability)

    def set_verbose(self, verbose):
        """
        Set to True to print the HTTP requests.
        :param verbose
        """
        self._debug = verbose

    def set_couples_accepted(self, couples_accepted):
        """
        Set to true to only return listings that accept couples.
        :param couples_accepted:
        """
        if couples_accepted:
            self._query_params += str(QueryParam.COUPLES_ACCEPTED)

    def set_gender(self, gender_type):
        self._query_params += str(QueryParam.GENDER) + str(gender_type)

    def set_ensuite_only(self, ensuite_only):
        """
        Set to true to only return listings that are ensuite only.
        :param ensuite_only:
        """
        if ensuite_only:
            self._query_params += str(QueryParam.ENSUITE_ONLY)

    def set_room_type(self, room_type):
        """
        Set the room type.
        :param room_type:
        """
        if not isinstance(room_type, RoomType):
            raise DaftException("room_type should be an instance of RoomType.")
        self._query_params += str(QueryParam.ROOM_TYPE) + str(room_type)

    def set_with_photos(self, with_photos):
        """
        Set to True to only get listings that has photos.
        :param with_photos
        """
        if with_photos:
            self._query_params += str(QueryParam.WITH_PHOTOS)

    def set_keywords(self, keywords):
        """
        Pass an array to filter the result by keywords.
        :param keywords
        """
        self._query_params += str(QueryParam.KEYWORDS) + '+'.join(keywords)

    def set_furnished(self, furnished):
        """
        Set to true to only get rental properties that are furnished.
        :param furnished:
        :return:
        """
        if furnished:
            self._query_params += str(QueryParam.FURNISHED)

    def set_area(self, area):
        """
        The area to retrieve listings from. Use an array to search multiple areas.
        :param area:
        :return:
        """
        self._area = area.replace(" ", "-").lower() if isinstance(area, str) else ','.join(
            map(lambda x: x.lower().replace(' ', '-'), area))

    def set_county(self, county):
        """
        The county to retrieve listings from.
        :param county:
        :return:
        """
        self._county = county.replace(" ", "-").lower()

    def set_open_viewing(self, open_viewing):
        """
        Set to True to only search for properties that have upcoming 'open for viewing' dates.
        :param open_viewing:
        :return:
        """
        if open_viewing:
            self._open_viewing = open_viewing
            self._query_params += str(QueryParam.OPEN_VIEWING)

    def set_offset(self, offset):
        """
        The page number which is in increments of 10. The default page number is 0.
        :param offset:
        :return:
        """

        if not isinstance(offset, int) or offset < 0:
            raise DaftException("Offset should be a positive integer.")

        self._offset = str(offset)

    def set_min_price(self, min_price):
        """
        The minimum price.
        :param min_price:
        :return:
        """

        if not isinstance(min_price, int):
            raise DaftException("Min price should be an integer.")

        self._min_price = str(min_price)
        self._price += str(QueryParam.MIN_PRICE) + self._min_price

    def set_max_price(self, max_price):
        """
        The maximum price.
        :param max_price:
        :return:
        """

        if not isinstance(max_price, int):
            raise DaftException("Max price should be an integer.")

        self._max_price = str(max_price)
        self._price += str(QueryParam.MAX_PRICE) + self._max_price

    def set_listing_type(self, listing_type):
        """
        The listings you'd like to scrape i.e houses, properties, auction, commercial or apartments.
        Use the SaleType or RentType enum to select the listing type.
        i.e set_listing_type(SaleType.PROPERTIES)
        :param listing_type:
        :return:
        """

        if not isinstance(listing_type, SaleType) and not isinstance(listing_type, RentType):
            raise DaftException("listing_type should be an instance of SaleType or RentType.")

        self._listing_type = listing_type

    def set_sale_agreed(self, sale_agreed):
        """
        If set to True, we'll scrape listings that are sale agreed.
        :param sale_agreed:
        :return:
        """
        self._sale_agreed = sale_agreed

    def set_min_beds(self, min_beds):
        """
        The minimum number of beds.
        :param min_beds:
        :return:
        """

        if not isinstance(min_beds, int):
            raise DaftException("Minimum number of beds should be an integer.")

        self._min_beds = str(min_beds)
        self._query_params += str(QueryParam.MIN_BEDS) + self._min_beds

    def set_max_beds(self, max_beds):
        """
        The maximum number of beds.
        :param max_beds:
        :return:
        """
        if not isinstance(max_beds, int):
            raise DaftException("Maximum number of beds should be an integer.")

        self._max_beds = str(max_beds)
        self._query_params += str(QueryParam.MAX_BEDS) + self._max_beds

    def set_sort_by(self, sort_by):
        """
        Use this method to sort by price, distance, upcoming viewing or date using the SortType object.
        :param sort_by:
        :return:
        """
        if not isinstance(sort_by, SortType):
            raise DaftException("sort_by should be an instance of SortType.")

        self._sort_by = str(sort_by)

    def set_sort_order(self, sort_order):
        """
        Use the SortOrder object to sort the listings descending or ascending.
        :param sort_order:
        :return:
        """

        if not isinstance(sort_order, SortOrder):
            raise DaftException("sort_order should be an instance of SortOrder.")

        self._sort_order = str(sort_order)

    def set_commercial_property_type(self, commercial_property_type):
        """
        Use the CommercialType object to set the commercial property type.
        :param commercial_property_type:
        :return:
        """

        if not isinstance(commercial_property_type, CommercialType):
            raise DaftException("commercial_property_type should be an instance of CommercialType.")

        self._commercial_property_type = str(commercial_property_type)

    def set_commercial_min_size(self, commercial_min_size):
        """
        The minimum size in sq ft.
        :param commercial_min_size:
        :return:
        """
        if not isinstance(commercial_min_size, int):
            raise DaftException("commercial_min_size should be an integer.")

        self._commercial_min_size = str(commercial_min_size)
        self._query_params += str(QueryParam.COMMERCIAL_MIN) + self._commercial_min_size

    def set_commercial_max_size(self, commercial_max_size):
        """
        The maximum size in sq ft.
        :param commercial_max_size:
        :return:
        """
        if not isinstance(commercial_max_size, int):
            raise DaftException("commercial_max_size should be an integer.")

        self._commercial_max_size = str(commercial_max_size)
        self._query_params += str(QueryParam.COMMERCIAL_MAX) + self._commercial_max_size

    def set_university(self, university):
        """
        Set the university.
        :param university: University
        :return:
        """
        self._university = str(university)

    def set_student_accommodation_type(self, student_accommodation_type):
        """
        Set the student accomodation type.
        :param student_accommodation_type: StudentAccomodationType
        """
        if not isinstance(student_accommodation_type, StudentAccommodationType):
            raise DaftException("student_accommodation_type should be an instance of StudentAccommodationType.")

        self._student_accommodation_type = str(student_accommodation_type)

    def set_num_occupants(self, num_occupants):
        """
        Set the max number of occupants living in the property for rent.
        :param num_occupants: int
        """
        self._query_params += str(QueryParam.NUM_OCCUPANTS) + str(num_occupants)

    def set_area_type(self, area_type):
        """
        Set the area type.
        :param area_type: AreaType
        """
        self._query_params += str(area_type)

    def set_public_transport_route(self, public_transport_route):
        """
        Set the public transport route.
        :param public_transport_route: TransportRoute
        """
        self._query_params += str(QueryParam.ROUTE_ID) + str(public_transport_route)

    def set_property_type(self, property_types):
        """
        Set the property type for rents.
        :param property_types: Array of Enum PropertyType
        """
        query_add = ''
        for property_type in property_types:
            if not isinstance(property_type, PropertyType):
                raise DaftException("property_types should be an instance of PropertyType.")
            query_add += str(property_type)

        self._query_params += query_add

    def set_xml_url(self, xml_url):
        """
        Set the url from xml to loop
        :param set_xml_url: String with url
        """
        self._xml_url = xml_url

    def get_url(self):

        if self._result_url:
            if self._offset:
                self._result_url += '&offset=' + str(self._offset)
            return self._result_url

        if self._sort_by:
            if self._sort_order:
                self._query_params += str(QueryParam.SORT_ORDER) + str(self._sort_order)
                self._query_params += str(QueryParam.SORT_BY) + str(self._sort_by)
            else:
                self._query_params += str(QueryParam.SORT_ORDER) + str(SortOrder.DESCENDING)
                self._query_params += str(QueryParam.SORT_BY) + self._sort_by

        if self._university and isinstance(self._listing_type, RentType):
            if self._min_price or self._max_price:
                self._query_params += self._price

            url = self._base + str(
                self._listing_type) + self._university + self._student_accommodation_type + '?' + self._query_params
            return url

        # If the county is not set then we'll look at properties throughout Ireland.
        if self._county is None:
            self._county = 'ireland'

        if self._area is None:
            self._area = ''

        if self._sale_agreed:
            if self._min_price or self._max_price:
                self._query_params += self._price + str(QueryParam.SALE_AGREED_WITH_PRICE)
            else:
                self._query_params += str(QueryParam.SALE_AGREED)
        else:
            if self._min_price or self._max_price:
                self._query_params += self._price

        if self._min_price or self._max_price and isinstance(self._listing_type, RentType):
            self._query_params += str(QueryParam.IGNORED_AGENTS)

        url = self._base + self._county + str(self._listing_type) + str(self._commercial_property_type) + str(
            self._area) + '?offset=' + str(self._offset) + self._query_params
        return url

    def search(self):
        """
        The search function returns an array of Listing objects.
        :return: Listing object
        """
        listings = []
        request = Request(debug=self._debug)
        url = self.get_url()
        soup = request.get(url)
        divs = soup.find_all("div", {"class": "box"})
        [listings.append(Listing(div, self._debug)) for div in divs]
        return listings

    def read_xml(self, xml_url=None):
        if xml_url:
            self._xml_url = xml_url
        listings = []
        request = Request(debug=self._debug)
        soup = request.get(self._xml_url)
        divs = soup.find_all("item")
        [listings.append(Listing(div.find('guid').text, self._debug)) for div in divs]
        return listings



import base64
import logging
import re

import html2text

from .request import Request


class Listing(object):
    def __init__(self,
                 data_from_search=None,
                 url=None,
                 debug=False):

        if isinstance(data_from_search, str):
            from bs4 import BeautifulSoup
            soup = BeautifulSoup(data_from_search)
            data_from_search = soup.div

        self._data_from_search = data_from_search
        self._url = url
        self._debug = debug
        self._ad_page_content_data = None

    @property
    def _ad_page_content(self):
        if self._ad_page_content_data is not None:
            return self._ad_page_content_data

        if self._url:
            self._ad_page_content_data = Request(
                debug=self._debug).get(self._url)
        else:
            self._ad_page_content_data = Request(
                debug=self._debug).get(self.daft_link)

        return self._ad_page_content_data

    @property
    def id(self):
        try:
            return self._ad_page_content.find('input', {'id': 'ad_id'})['value']
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting id. Error message: " + e.message)
            return

    @property
    def description(self):
        try:
            description_div = str(
                self._ad_page_content.find('div', {'id': 'description'})
            )

            pos_token = description_div.find('<!-- dont_cut_below_here -->')
            if pos_token == -1:
                return None
            return html2text.html2text(description_div[0:pos_token])
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting description. Error message: " + e.message)
            return

    @property
    def agent_id(self):
        try:
            return self._ad_page_content.find('input', {'id': 'agent_id'})['value']
        except Exception as e:
            if self._debug:
                logging.error("Error getting agent_id. Error message: " + e.message)
            return

    @property
    def search_type(self):
        try:
            return self._ad_page_content.find('input', {'id': 'ad_search_type'})['value']
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting search_type. Error message: " + e.message)
            return

    @property
    def price(self):
        """
        This method returns the price.
        :return:
        """
        try:
            if self._data_from_search:
                return self._data_from_search.find('strong', {'class': 'price'}).text
            else:
                return self._ad_page_content.find('div', {'id': 'smi-price-string'}).text
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting price. Error message: " + e.message)
            return

    @property
    def price_change(self):
        """
        This method returns any price change.
        :return:
        """
        try:
            if self._data_from_search:
                return self._data_from_search.find('div', {'class': 'price-changes-sr'}).text
            else:
                return self._ad_page_content.find('div', {'class': 'price-changes-sr'}).text
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting price_change. Error message: " + e.message)
            return

    @property
    def upcoming_viewings(self):
        """
        Returns an array of upcoming viewings for a property.
        :return:
        """
        upcoming_viewings = []
        try:
            if self._data_from_search:
                viewings = self._data_from_search.find_all(
                    'div', {'class': 'smi-onview-text'})
            else:
                viewings = []
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting upcoming_viewings. Error message: " + e.message)
            return
        for viewing in viewings:
            upcoming_viewings.append(viewing.text.strip())
        return upcoming_viewings

    @property
    def facilities(self):
        """
        This method returns the properties facilities.
        :return:
        """
        facilities = []
        try:
            list_items = self._ad_page_content.select("#facilities li")
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting facilities. Error message: " + e.message)
            return

        for li in list_items:
            facilities.append(li.text)
        return facilities

    @property
    def overviews(self):
        """
        This method returns the properties overviews.
        :return:
        """
        overviews = []
        try:
            list_items = self._ad_page_content.select("#overview li")
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting overviews. Error message: " + e.message)
            return

        for li in list_items:
            overviews.append(li.text)
        return overviews

    @property
    def features(self):
        """
        This method returns the properties features.
        :return:
        """
        features = []
        try:
            list_items = self._ad_page_content.select("#features li")
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting features. Error message: " + e.message)
            return

        for li in list_items:
            features.append(li.text)
        return features

    @property
    def formalised_address(self):
        """
        This method returns the formalised address.
        :return:
        """
        try:
            if self._data_from_search:
                t = self._data_from_search.find('a').contents[0]
            else:
                t = self._ad_page_content.find(
                    'div', {'class': 'smi-object-header'}).find(
                    'h1').text.strip()

        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting formalised_address. Error message: " + e.message)
            return
        s = t.split('-')
        a = s[0].strip()
        if 'SALE AGREED' in a:
            a = a.split()
            a = a[3:]
            a = ' '.join([str(x) for x in a])
        return a.lower().title().strip()

    @property
    def address_line_1(self):
        """
        This method returns the first line of the address.
        :return:
        """
        formalised_address = self.formalised_address
        if formalised_address is None:
            return
        try:
            address = formalised_address.split(',')
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting address_line_1. Error message: " + e.message)
            return

        return address[0].strip()

    @property
    def county(self):
        """
        This method returns the county name.
        :return:
        """
        formalised_address = self.formalised_address

        if formalised_address is None:
            return

        try:
            address = formalised_address.split(',')
            return address[-1].strip()
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting county. Error message: " + e.message)
            return

    @property
    def images(self):
        """
        This method returns the listing image.
        :return:
        """
        try:
            uls = self._ad_page_content.find(
                "ul", {"class": "smi-gallery-list"})
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting images. Error message: " + e.message)
            return
        images = []
        if uls is None:
            return
        for li in uls.find_all('li'):
            if li.find('img')['src']:
                images.append(li.find('img')['src'])

        return images

    @property
    def hires_images(self):
        """
        This method returns the listing big image.
        :return:
        """
        try:
            uls = self._ad_page_content.find("div", {"id": "pbxl_carousel"})
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting hires_image. Error message: " + e.message)
            return
        hires_images = []
        if uls is None:
            return
        for li in uls.find_all("li", {"class": "pbxl_carousel_item"}):
            if li.find('img')['src']:
                hires_images.append(li.find('img')['src'])

        return hires_images

    @property
    def agent(self):
        """
        This method returns the agent name.
        :return:
        """
        try:
            if self._data_from_search:
                agent = self._data_from_search.find(
                    'ul', {'class': 'links'}).text
                return agent.split(':')[1].strip()
            else:
                return self._ad_page_content.find('a', {'id': 'smi-link-branded'}).text.strip()
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting agent. Error message: " + e.message)
            return

    @property
    def agent_url(self):
        """
        This method returns the agent's url.
        :return:
        """
        try:
            if self._data_from_search:
                agent = self._data_from_search.find('ul', {'class': 'links'})
                links = agent.find_all('a')
                return links[1]['href']
            else:
                return self._ad_page_content.find('a', {'id': 'smi-link-branded'})['href']
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting agent_url. Error message: " + e.message)
            return

    @property
    def contact_number(self):
        """
        This method returns the contact phone number.
        :return:
        """
        try:
            number = self._ad_page_content.find(
                'button', {'class': 'phone-number'})
            return (base64.b64decode(number.attrs['data-p'])).decode('ascii')
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting contact_number. Error message: " + e.message)
            return 'N/A'

    @property
    def daft_link(self):
        """
        This method returns the url of the listing.
        :return:
        """
        try:
            if self._data_from_search:
                link = self._data_from_search.find('a', href=True)
                return 'http://www.daft.ie' + link['href']
            else:
                return self._ad_page_content.find('link', {'rel': 'canonical'})['href']
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting daft_link. Error message: " + e.message)
            return

    @property
    def shortcode(self):
        """
        This method returns the shortcode url of the listing.
        :return:
        """
        try:
            div = self._ad_page_content.find(
                'div', {'class': 'description_extras'})
            index = [i for i, s in enumerate(
                div.contents) if 'Shortcode' in str(s)][0] + 1
            return div.contents[index]['href']
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting shortcode. Error message: " + e.message)
            return 'N/A'

    @property
    def date_insert_update(self):
        """
        This method returns the shortcode url of the listing.
        :return:
        """
        try:
            div = self._ad_page_content.find(
                'div', {'class': 'description_extras'})
            index = [i for i, s in enumerate(
                div.contents) if 'Entered/Renewed' in str(s)][0] + 1
            return re.search("([0-9]{1,2}/[0-9]{1,2}/[0-9]{4})", str(div.contents[index]))[0]
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting date_insert_update. Error message: " + e.message)
            return 'N/A'

    @property
    def views(self):
        """
        This method returns the "Property Views" from listing.
        :return:
        """
        try:
            div = self._ad_page_content.find(
                'div', {'class': 'description_extras'})
            index = [i for i, s in enumerate(
                div.contents) if 'Property Views' in str(s)][0] + 1
            return int(''.join(list(filter(str.isdigit, div.contents[index]))))
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting views. Error message: " + e.message)
            return 'N/A'

    @property
    def dwelling_type(self):
        """
        This method returns the dwelling type.
        :return:
        """
        try:
            if self._data_from_search:
                info = self._data_from_search.find(
                    'ul', {"class": "info"}).text
                s = info.split('|')
                return s[0].strip()
            else:
                return self._ad_page_content.find(
                    'div', {'id': 'smi-summary-items'}
                ).find('span', {'class': 'header_text'}).text

        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting dwelling_type. Error message: " + e.message)
            return

    @property
    def posted_since(self):
        """
        This method returns the date the listing was entered.
        :return:
        """
        try:
            if self._data_from_search:
                info = self._data_from_search.find(
                    'div', {"class": "date_entered"}).text
                s = info.split(':')
                return s[-1].strip()
            else:
                div = self._ad_page_content.find(
                    'div', {'class': 'description_extras'})
                index = [i for i, s in enumerate(
                    div.contents) if 'Entered/Renewed' in str(s)][0] + 1
                return re.search("([0-9]{1,2}/[0-9]{1,2}/[0-9]{4})", str(div.contents[index]))[0]
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting posted_since. Error message: " + e.message)
            return

    @property
    def bedrooms(self):
        """
        This method gets the number of bedrooms.
        :return:
        """
        try:
            if self._data_from_search:
                info = self._data_from_search.find(
                    'ul', {"class": "info"}).text
                s = info.split('|')
                nb = s[1].strip()
                return int(nb.split()[0])
            else:
                div = self._ad_page_content.find(
                    'div', {'id': 'smi-summary-items'})
                spans = div.find_all('span', {'class': 'header_text'})
                for span in spans:
                    # print(span.text)
                    if 'bed' in span.text.lower():
                        return int(''.join([n for n in span.text if n.isdigit()]))
                return
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting bedrooms. Error message: " + e.message)
            return 'N/A'

    @property
    def bathrooms(self):
        """
        This method gets the number of bathrooms.
        :return:
        """
        try:
            if self._data_from_search:
                info = self._data_from_search.find(
                    'ul', {"class": "info"}).text
                s = info.split('|')
                nb = s[2].strip()
                return int(nb.split()[0])
            else:
                div = self._ad_page_content.find(
                    'div', {'id': 'smi-summary-items'})
                spans = div.find_all('span', {'class': 'header_text'})
                for span in spans:
                    # print(span.text)
                    if 'bath' in span.text.lower():
                        return int(''.join([n for n in span.text if n.isdigit()]))
                return

        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting bathrooms. Error message: " + e.message)
            return 'N/A'

    @property
    def city_center_distance(self):
        """
        This method gets the distance to city center, in km.
        :return:
        """
        try:
            infos = self._ad_page_content.find_all(
                'div', {"class": "map_info_box"})
            for info in infos:
                if 'Distance to City Centre' in info.text:
                    distance_list = re.findall(
                        'Distance to City Centre: (.*) km', info.text)
                    return distance_list[0]
            return None
        except Exception as e:
            if self._debug:
                logging.error(e.message)
            print(e)
            return 'N/A'

    @property
    def transport_routes(self):
        """
        This method gets a dict of routes listed in Daft.
        :return:
        """
        routes = {}
        try:
            big_div = self._ad_page_content.find(
                'div', {"class": "half_area_box_right"})
            uls = big_div.find("ul")
            if uls is None:
                return None
            for li in uls.find_all('li'):
                route_li = li.text.split(':')
                routes[route_li[0]] = [x.strip()
                                       for x in route_li[1].split(',')]
            return routes
        except Exception as e:
            if self._debug:
                logging.error(e.message)
            return 'N/A'

    @property
    def latitude(self):
        """
        This method gets a dict of routes listed in Daft.
        :return:
        """
        try:
            scripts = self._ad_page_content.find_all('script')
            for script in scripts:
                if 'latitude' in script.text:
                    find_list = re.findall(
                        r'"latitude":"([\-]?[0-9.]*[0-9]+)"', script.text)
                    if len(find_list) >= 1:
                        return find_list[0]
            return None
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting latitude. Error message: " + e.message)
            return None

    @property
    def longitude(self):
        """
        This method gets a dict of routes listed in Daft.
        :return:
        """
        try:
            scripts = self._ad_page_content.find_all('script')
            for script in scripts:
                if 'longitude' in script.text:
                    find_list = re.findall(
                        r'"longitude":"([\-]?[0-9.]*[0-9]+)"', script.text)
                    if len(find_list) >= 1:
                        return find_list[0]
            return None
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting longitude. Error message: " + e.message)
            return None

    @property
    def ber_code(self):
        """
        This method gets ber code listed in Daft.
        :return:
        """
        try:
            alt_text = self._ad_page_content.find(
                'span', {'class': 'ber-hover'}
            ).find('img')['alt']

            if ('exempt' in alt_text):
                return 'exempt'
            else:
                alt_arr = alt_text.split()
                if 'ber' in alt_arr[0].lower():
                    return alt_arr[1].lower()
                else:
                    return None
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting the Ber Code. Error message: " + e.message)
            return None

    @property
    def commercial_area_size(self):
        """
        This method returns the area size. This method should only be called when retrieving commercial type listings.
        :return:
        """
        try:
            if self._data_from_search:
                info = self._data_from_search.find(
                    'ul', {"class": "info"}).text
                s = info.split('|')
                return s[1].strip()
            else:
                return
        except Exception as e:
            if self._debug:
                logging.error(
                    "Error getting commercial_area_size. Error message: " + e.message)
            return 'N/A'

    def contact_advertiser(self, name, email, contact_number, message):
        """
        This method allows you to contact the advertiser of a listing.
        :param name: Your name
        :param email: Your email address.
        :param contact_number: Your contact number.
        :param message: Your message.
        :return: 
        """

        req = Request(debug=self._debug)

        ad_search_type = self.search_type
        agent_id = self.agent_id
        ad_id = self.id

        response = req.post('https://www.daft.ie/ajax_endpoint.php?', params={
            'action': 'daft_contact_advertiser',
            'from': name,
            'email': email,
            'message': message,
            'contact_number': contact_number,
            'type': ad_search_type,
            'agent_id': agent_id,
            'id': ad_id
        })

        if self._debug:
            logging.info("Status code: %d" % response.status_code)
            logging.info("Response: %s" % response.content)
        if response.status_code != 200:
            logging.error("Status code: %d" % response.status_code)
            logging.error("Response: %s" % response.content)
        return response.status_code == 200

    def as_dict(self):
        """
        Return a Listing object as Dictionary
        :return: dict
        """
        return {
            'search_type': self.search_type,
            'agent_id': self.agent_id,
            'id': self.id,
            'price': self.price,
            'price_change': self.price_change,
            'viewings': self.upcoming_viewings,
            'facilities': self.facilities,
            'overviews': self.overviews,
            'formalised_address': self.formalised_address,
            'address_line_1': self.address_line_1,
            'county': self.county,
            'listing_image': self.images,
            'listing_hires_image': self.hires_images,
            'agent': self.agent,
            'agent_url': self.agent_url,
            'contact_number': self.contact_number,
            'daft_link': self.daft_link,
            'shortcode': self.shortcode,
            'date_insert_update': self.date_insert_update,
            'views': self.views,
            'description': self.description,
            'dwelling_type': self.dwelling_type,
            'posted_since': self.posted_since,
            'num_bedrooms': self.bedrooms,
            'num_bathrooms': self.bathrooms,
            'city_center_distance': self.city_center_distance,
            'transport_routes': self.transport_routes,
            'latitude': self.latitude,
            'longitude': self.longitude,
            'ber_code': self.ber_code,
            'commercial_area_size': self.commercial_area_size
        }

    def __repr__(self):
        return "Listing (%s)" % self.formalised_address



import logging

import requests
from bs4 import BeautifulSoup

from .exceptions import DaftException


class Request:
    def __init__(self, debug=False):
        self._headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.0; WOW64; rv:24.0) Gecko/20100101 Firefox/24.0'}
        self._debug = debug

    def get(self, url, params=None):
        req = requests.get(url, headers=self._headers, params=params)
        if self._debug:
            logging.info("URL: " + req.url)
            logging.info("Status code: " + str(req.status_code))

        if req.status_code != 200:
            raise DaftException(reason=req.reason)

        soup = BeautifulSoup(req.content, 'html.parser')
        return soup

    def post(self, url, params=None):
        req = requests.post(url, params=params, headers=self._headers)

        if self._debug:
            logging.info("URL: " + req.url)
            logging.info("Status code: " + str(req.status_code))

        if req.status_code != 200:
            raise DaftException(reason=req.reason)

        return req



from enum import Enum


class PropertyType(Enum):
    APARTMENT = '&s%5Bpt_id%5D%5B0%5D=1'
    HOUSE = '&s%5Bpt_id%5D%5B1%5D=2'
    STUDIO = '&s%5Bpt_id%5D%5B2%5D=3'
    FLAT = '&s%5Bpt_id%5D%5B3%5D=4'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SaleType: %s>" % self


class SaleType(Enum):
    HOUSES = '/houses-for-sale/'
    PROPERTIES = '/property-for-sale/'
    AUCTION = '/houses-for-auction/'
    APARTMENTS = '/apartments-for-sale/'
    COMMERCIAL = '/commercial-property/'
    NEW = '/new-homes-for-sale/'
    OVERSEAS = '/overseas-property-for-sale/'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SaleType: %s>" % self


class CommercialType(Enum):
    OFFICE = '/offices/'
    RETAIL = '/retail-units/'
    OFFICE_SHARE = '/office-share/'
    INDUSTRIAL_UNIT = '/industrial-unit/'
    COMMERCIAL_SITE = '/commercial-site/'
    AGRICULTURAL_LAND = '/agricultural-farm-land/'
    RESTAURANT_BAR_HOTEL = '/restaurant-hotel-bar/'
    INDUSTRIAL_SITE = '/industrial_site/'
    DEV_LAND = '/development-land/'
    INVESTMENT_PROPERTY = '/investment-property/'
    SERVICED_OFFICE = '/serviced-office/'
    OVERSEAS = '/overseas-commercial-property-for-sale/'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<CommercialType: %s>" % self


class RentType(Enum):
    HOUSES = '/houses-for-rent/'
    APARTMENTS = '/apartments-for-rent/'
    ANY = '/residential-property-for-rent/'
    STUDIO = '/studio-apartments-for-rent/'
    FLAT = '/flats-for-rent/'
    SHORT_TERM = '/short-term-rentals/'
    STUDENT_ACCOMMODATION = '/student-accommodation/'
    PARKING_SPACES = '/parking-spaces/'
    ROOMS_TO_SHARE = '/rooms-to-share/'
    FLAT_TO_SHARE = '/flat-to-share/'
    APARTMENT_TO_SHARE = '/apartment-share/'
    HOUSE_SHARE = '/house-share/'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<RentType: %s>" % self


class StudentAccommodationType(Enum):
    ROOMS_TO_SHARE = '/rooms-to-share/'
    APARTMENTS = '/apartments-for-rent/'
    ANY = '/residential-property-for-rent/'
    STUDIO = '/studio-apartments-for-rent/'
    FLAT = '/flats-for-rent/'
    SHORT_TERM = '/short-term-rentals/'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<StudentAccommodationType: %s>" % self


class RoomType(Enum):
    SINGLE = 'single'
    DOUBLE = 'double'
    TWIN_ROOM = 'twin'
    SHARED = 'shared'
    SINGLE_OR_DOUBLE = 'own'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<RoomType: %s>" % self


class QueryParam(Enum):
    SALE_AGREED = '&s[area_type]=on&s[agreed]=1&s[advanced]=1'
    SALE_AGREED_WITH_PRICE = '&s%5Bagreed%5D=1&s%5Badvanced%5D=1'
    MIN_PRICE = '&s%5Bmnp%5D='
    MAX_PRICE = '&s%5Bmxp%5D='
    IGNORED_AGENTS = '&s%5Bignored_agents%5D%5B1%5D'
    AVALIABILITY = '&s%5Bavailable_for%5D='
    MIN_BEDS = '&s%5Bmnb%5D='
    MAX_BEDS = '&s%5Bmxb%5D='
    SORT_BY = '&s%5Bsort_by%5D='
    SORT_ORDER = '&s%5Bsort_type%5D='
    COMMERCIAL_MIN = '&s%5Bmin_size%5D='
    COMMERCIAL_MAX = '&s%5Bmax_size%5D='
    OPEN_VIEWING = '&s%5Bopenviewing%5D=1'
    WITH_PHOTOS = '&s%5Bphotos%5D=1'
    KEYWORDS = '&s%5Btxt%5D='
    FURNISHED = '&s%5Bfurn%5D=1'
    COUPLES_ACCEPTED = '&s%5Bcouples%5D=1'
    ENSUITE_ONLY = '&s%5Bes%5D=1'
    ROOM_TYPE = '&s%5Broom_type%5D='
    GENDER = '&s%5Bgender%5D='
    ADDRESS = '&s%5Baddress%5D='
    ADVANCED = '&s%5Badvanced%5D=1'
    MIN_LEASE = '&s%5Bmin_lease%5D='
    MAX_LEASE = '&s%5Bmax_lease%5D='
    DAYS_OLD = '&s%5Bdays_old%5D='
    NUM_OCCUPANTS = '&s%5Boccupants%5D='
    ROUTE_ID = '&s%5Broute_id%5D='
    FIND_TEAMUPS = "&submit_search=Find+Teamups+%BB"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<QueryParam: %s>" % self


class Gender(Enum):
    MALE = 'male'
    FEMALE = 'female'
    EITHER = 'on'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<Gender: %s>" % self


class SortOrder(Enum):
    ASCENDING = 'a'
    DESCENDING = 'd'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SortOrder: %s>" % self


class SortType(Enum):
    DATE = 'date'
    DISTANCE = 'distance'
    PRICE = 'price'
    UPCOMING_VIEWING = 'upcoming_viewing'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<SortType: %s>" % self


class University(Enum):
    NCI = 'national-college-of-ireland-nci'
    UCD = 'university-college-dublin-nui'
    GCD = 'griffith-college-dublin'
    TCD = 'trinity-college-university-of-dublin'
    DCU = 'dublin-city-university'
    DIT = 'dit-kevin-street'
    WIT = 'waterford-institute-of-technology'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<University: %s>" % self


class AreaType(Enum):
    ENROUTE = '&s%5Barea_type%5D=enroute'
    TRANSPORT_ROUTE = '&s%5Barea_type%5D=trans'

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<AreaType: %s>" % self


class TransportRoute(Enum):
    DART = "1"
    LUAS_TALLAGHT_LINE = "2"
    LUAS_SANDYFORD_LINE = "3"
    BUS_LINE_1 = "4"
    BUS_LINE_2 = "5"
    BUS_LINE_3 = "224"
    BUS_LINE_4 = "227"
    BUS_LINE_5 = "6"
    BUS_LINE_7 = "7"
    BUS_LINE_7A = "8"
    BUS_LINE_7B = "9"
    BUS_LINE_7D = "10"
    BUS_LINE_7N = "11"
    BUS_LINE_8 = "225"
    BUS_LINE_9 = "228"
    BUS_LINE_10 = "12"
    BUS_LINE_10A = "13"
    BUS_LINE_11 = "14"
    BUS_LINE_11A = "15"
    BUS_LINE_11B = "16"
    BUS_LINE_13 = "17"
    BUS_LINE_13A = "18"
    BUS_LINE_13B = "19"
    BUS_LINE_14 = "20"
    BUS_LINE_15 = "22"
    BUS_LINE_15A = "23"
    BUS_LINE_15B = "24"
    BUS_LINE_15C = "25"
    BUS_LINE_15D = "26"
    BUS_LINE_15E = "27"
    BUS_LINE_15F = "28"
    BUS_LINE_15N = "29"
    BUS_LINE_15X = "30"
    BUS_LINE_16 = "31"
    BUS_LINE_17 = "33"
    BUS_LINE_17A = "34"
    BUS_LINE_18 = "35"
    BUS_LINE_19 = "36"
    BUS_LINE_19A = "37"
    BUS_LINE_20B = "38"
    BUS_LINE_25 = "39"
    BUS_LINE_25A = "40"
    BUS_LINE_25N = "41"
    BUS_LINE_25X = "42"
    BUS_LINE_26 = "43"
    BUS_LINE_27 = "44"
    BUS_LINE_27B = "45"
    BUS_LINE_27C = "46"
    BUS_LINE_27N = "47"
    BUS_LINE_27X = "48"
    BUS_LINE_29A = "49"
    BUS_LINE_29N = "50"
    BUS_LINE_31 = "51"
    BUS_LINE_31A = "52"
    BUS_LINE_31B = "53"
    BUS_LINE_31N = "54"
    BUS_LINE_32 = "55"
    BUS_LINE_32A = "56"
    BUS_LINE_32B = "57"
    BUS_LINE_32X = "58"
    BUS_LINE_33 = "59"
    BUS_LINE_33A = "60"
    BUS_LINE_33B = "61"
    BUS_LINE_33N = "62"
    BUS_LINE_37 = "63"
    BUS_LINE_37X = "64"
    BUS_LINE_38 = "65"
    BUS_LINE_38A = "66"
    BUS_LINE_38B = "67"
    BUS_LINE_38C = "68"
    BUS_LINE_39 = "69"
    BUS_LINE_39A = "70"
    BUS_LINE_39B = "71"
    BUS_LINE_39N = "72"
    BUS_LINE_39X = "73"
    BUS_LINE_40 = "74"
    BUS_LINE_40A = "75"
    BUS_LINE_40B = "76"
    BUS_LINE_40C = "77"
    BUS_LINE_40D = "78"
    BUS_LINE_40N = "79"
    BUS_LINE_41 = "80"
    BUS_LINE_41A = "81"
    BUS_LINE_41B = "82"
    BUS_LINE_41C = "83"
    BUS_LINE_41N = "84"
    BUS_LINE_41X = "85"
    BUS_LINE_42 = "86"
    BUS_LINE_42A = "87"
    BUS_LINE_42B = "88"
    BUS_LINE_42N = "89"
    BUS_LINE_43 = "90"
    BUS_LINE_44 = "91"
    BUS_LINE_44B = "92"
    BUS_LINE_44C = "93"
    BUS_LINE_44N = "94"
    BUS_LINE_45 = "95"
    BUS_LINE_45A = "96"
    BUS_LINE_46 = "97"
    BUS_LINE_46A = "98"
    BUS_LINE_46B = "99"
    BUS_LINE_46C = "100"
    BUS_LINE_46D = "101"
    BUS_LINE_46E = "102"
    BUS_LINE_46N = "103"
    BUS_LINE_46X = "104"
    BUS_LINE_48A = "105"
    BUS_LINE_48N = "106"
    BUS_LINE_49 = "107"
    BUS_LINE_49A = "108"
    BUS_LINE_49N = "109"
    BUS_LINE_49X = "110"
    BUS_LINE_50 = "111"
    BUS_LINE_50X = "112"
    BUS_LINE_51 = "113"
    BUS_LINE_51A = "114"
    BUS_LINE_51B = "115"
    BUS_LINE_51C = "116"
    BUS_LINE_51D = "117"
    BUS_LINE_51N = "118"
    BUS_LINE_51X = "119"
    BUS_LINE_53 = "120"
    BUS_LINE_53A = "121"
    BUS_LINE_54A = "122"
    BUS_LINE_54N = "123"
    BUS_LINE_56A = "124"
    BUS_LINE_58C = "125"
    BUS_LINE_58X = "126"
    BUS_LINE_59 = "127"
    BUS_LINE_63 = "128"
    BUS_LINE_65 = "129"
    BUS_LINE_65B = "130"
    BUS_LINE_65X = "131"
    BUS_LINE_66 = "132"
    BUS_LINE_66A = "133"
    BUS_LINE_66B = "134"
    BUS_LINE_66N = "135"
    BUS_LINE_66X = "136"
    BUS_LINE_67 = "137"
    BUS_LINE_67A = "138"
    BUS_LINE_67N = "139"
    BUS_LINE_67X = "140"
    BUS_LINE_68 = "141"
    BUS_LINE_69 = "142"
    BUS_LINE_69N = "143"
    BUS_LINE_69X = "144"
    BUS_LINE_70 = "145"
    BUS_LINE_70N = "146"
    BUS_LINE_70X = "147"
    BUS_LINE_75 = "148"
    BUS_LINE_76 = "149"
    BUS_LINE_76A = "150"
    BUS_LINE_76B = "151"
    BUS_LINE_77 = "152"
    BUS_LINE_77A = "153"
    BUS_LINE_77B = "154"
    BUS_LINE_77N = "155"
    BUS_LINE_77X = "156"
    BUS_LINE_78 = "157"
    BUS_LINE_78A = "158"
    BUS_LINE_79 = "159"
    BUS_LINE_83 = "160"
    BUS_LINE_84 = "161"
    BUS_LINE_84N = "162"
    BUS_LINE_84X = "163"
    BUS_LINE_86 = "164"
    BUS_LINE_88N = "165"
    BUS_LINE_90 = "166"
    BUS_LINE_90A = "167"
    BUS_LINE_102 = "168"
    BUS_LINE_103 = "169"
    BUS_LINE_104 = "170"
    BUS_LINE_105 = "171"
    BUS_LINE_111 = "172"
    BUS_LINE_114 = "173"
    BUS_LINE_115 = "174"
    BUS_LINE_116 = "175"
    BUS_LINE_117 = "176"
    BUS_LINE_118 = "177"
    BUS_LINE_120 = "178"
    BUS_LINE_121 = "179"
    BUS_LINE_122 = "180"
    BUS_LINE_123 = "181"
    BUS_LINE_127 = "182"
    BUS_LINE_129 = "183"
    BUS_LINE_130 = "184"
    BUS_LINE_140 = "226"
    BUS_LINE_145 = "185"
    BUS_LINE_146 = "186"
    BUS_LINE_150 = "187"
    BUS_LINE_161 = "188"
    BUS_LINE_172 = "189"
    BUS_LINE_184 = "190"
    BUS_LINE_185 = "191"
    BUS_LINE_201 = "192"
    BUS_LINE_202 = "193"
    BUS_LINE_206 = "194"
    BUS_LINE_210 = "195"
    BUS_LINE_220 = "196"
    BUS_LINE_230 = "197"
    BUS_LINE_236 = "198"
    BUS_LINE_237 = "199"
    BUS_LINE_238 = "200"
    BUS_LINE_239 = "201"
    BUS_LINE_270 = "202"
    BUS_LINE_746 = "203"
    BUS_LINE_747 = "204"
    BUS_LINE_748 = "205"

    def __str__(self):
        return self._value_

    def __repr__(self):
        return "<TransportRoute: %s>" % self



# -*- coding: utf-8 -*-

import os
import sys
from shutil import rmtree

from setuptools import setup, Command

here = os.path.abspath(os.path.dirname(__file__))

with open("README.rst", "rb") as f:
    long_descr = f.read().decode("utf-8")


class PublishCommand(Command):
    """Support setup.py publish."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system('{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        sys.exit()


setup(name='daftlistings',
      version='1.7.0',
      description='A library that enables programmatic interaction with daft.ie. Daft.ie has nationwide coverage and contains about 80% of the total available properties in Ireland.',
      long_description=long_descr,
      url='https://github.com/AnthonyBloomer/daftlistings',
      author='Anthony Bloomer',
      keywords=['daft', 'web scraping', 'real estate', 'web scraper', 'daft.ie'],
      author_email='ant0@protonmail.ch',
      license='MIT',
      packages=['daftlistings'],
      install_requires=[
          'beautifulsoup4',
          'enum34',
          'requests'
      ],
      classifiers=[
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          "Topic :: Software Development :: Libraries",
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
      ],
      cmdclass={
          'publish': PublishCommand,
      },
      zip_safe=False)
